@@ -54,15 +54,14 @@ from verticapy.learn.plot import *
 from verticapy.utilities import *
 from verticapy.toolbox import *
 from verticapy import vDataFrame
-from verticapy.connections.connect import read_auto_connect
 from verticapy.errors import *
 from verticapy.learn.vmodel import *
 
 # ---#
-class MultinomialNB(MulticlassClassifier):
+class NaiveBayes(MulticlassClassifier):
     """
 ---------------------------------------------------------------------------
-Creates a MultinomialNB object by using the Vertica Highly Distributed 
+Creates a NaiveBayes object by using the Vertica Highly Distributed 
 and Scalable Naive Bayes on the data. It is a "probabilistic classifiers" 
 based on applying Bayes theorem with strong (naïve) independence assumptions 
 between the features. 
@@ -76,17 +75,79 @@ cursor: DBcursor, optional
 alpha: float, optional
 	A float that specifies use of Laplace smoothing if the event model is 
 	categorical, multinomial, or Bernoulli.
+nbtype: str, optional
+    Naive Bayes Type.
+    - auto        : Vertica NB will treat columns according to data type:
+        - FLOAT        : Values are assumed to follow some Gaussian 
+                         distribution.
+        - INTEGER      : Values are assumed to belong to one multinomial 
+                         distribution.
+        - CHAR/VARCHAR : Values are assumed to follow some categorical 
+                         distribution. The string values stored in these 
+                         columns must not be greater than 128 characters.
+        - BOOLEAN      : Values are treated as categorical with two values.
+     - bernoulli   : Casts the variables to boolean.
+     - categorical : Casts the variables to categorical.
+     - multinomial : Casts the variables to integer.
+     - gaussian    : Casts the variables to float.
 	"""
 
+    def __init__(
+        self, name: str, cursor=None, alpha: float = 1.0, nbtype: str = "auto"
+    ):
+        check_types(
+            [
+                ("name", name, [str],),
+                ("alpha", alpha, [int, float],),
+                (
+                    "nbtype",
+                    nbtype,
+                    ["auto", "bernoulli", "categorical", "multinomial", "gaussian"],
+                ),
+            ]
+        )
+        self.type, self.name = "NaiveBayes", name
+        self.set_params({"alpha": alpha, "nbtype": nbtype})
+        cursor = check_cursor(cursor)[0]
+        self.cursor = cursor
+        version(cursor=cursor, condition=[8, 0, 0])
+
+
+# ---#
+class BernoulliNB(NaiveBayes):
+    """i.e. NaiveBayes with param nbtype = 'bernoulli'"""
+
     def __init__(
         self, name: str, cursor=None, alpha: float = 1.0,
     ):
-        check_types([("name", name, [str], False)])
-        self.type, self.name = "MultinomialNB", name
-        self.set_params({"alpha": alpha})
-        if not (cursor):
-            cursor = read_auto_connect().cursor()
-        else:
-            check_cursor(cursor)
-        self.cursor = cursor
-        version(cursor=cursor, condition=[8, 0, 0])
+        super().__init__(name, cursor, alpha, "bernoulli")
+
+
+# ---#
+class CategoricalNB(NaiveBayes):
+    """i.e. NaiveBayes with param nbtype = 'categorical'"""
+
+    def __init__(
+        self, name: str, cursor=None, alpha: float = 1.0,
+    ):
+        super().__init__(name, cursor, alpha, "categorical")
+
+
+# ---#
+class GaussianNB(NaiveBayes):
+    """i.e. NaiveBayes with param nbtype = 'gaussian'"""
+
+    def __init__(
+        self, name: str, cursor=None,
+    ):
+        super().__init__(name, cursor, nbtype="gaussian")
+
+
+# ---#
+class MultinomialNB(NaiveBayes):
+    """i.e. NaiveBayes with param nbtype = 'multinomial'"""
+
+    def __init__(
+        self, name: str, cursor=None, alpha: float = 1.0,
+    ):
+        super().__init__(name, cursor, alpha, "multinomial")
