@@ -50,7 +50,6 @@
 #
 # VerticaPy Modules
 from verticapy.learn.vmodel import *
-from verticapy.connections.connect import read_auto_connect
 
 
 # ---#
@@ -66,7 +65,7 @@ name: str
 	Name of the the model. The model will be stored in the DB.
 cursor: DBcursor, optional
 	Vertica DB cursor.
-max_features: str, optional
+max_features: str/int, optional
 	The number of randomly chosen features from which to pick the best feature 
 	to split on a given tree node. It can be an integer or one of the two following
 	methods.
@@ -93,7 +92,7 @@ nbins: int, optional
         self,
         name: str,
         cursor=None,
-        max_features="auto",
+        max_features: (int, str) = "auto",
         max_leaf_nodes: int = 1e9,
         max_depth: int = 100,
         min_samples_leaf: int = 1,
@@ -114,10 +113,7 @@ nbins: int, optional
                 "nbins": nbins,
             }
         )
-        if not (cursor):
-            cursor = read_auto_connect().cursor()
-        else:
-            check_cursor(cursor)
+        cursor = check_cursor(cursor)[0]
         self.cursor = cursor
         version(cursor=cursor, condition=[8, 1, 1])
 
@@ -135,7 +131,7 @@ name: str
 	Name of the the model. The model will be stored in the DB.
 cursor: DBcursor, optional
 	Vertica DB cursor.
-max_features: str, optional
+max_features: str/int, optional
 	The number of randomly chosen features from which to pick the best feature 
 	to split on a given tree node. It can be an integer or one of the two following
 	methods.
@@ -162,7 +158,7 @@ nbins: int, optional
         self,
         name: str,
         cursor=None,
-        max_features="auto",
+        max_features: (int, str) = "auto",
         max_leaf_nodes: int = 1e9,
         max_depth: int = 100,
         min_samples_leaf: int = 1,
@@ -183,10 +179,7 @@ nbins: int, optional
                 "nbins": nbins,
             }
         )
-        if not (cursor):
-            cursor = read_auto_connect().cursor()
-        else:
-            check_cursor(cursor)
+        cursor = check_cursor(cursor)[0]
         self.cursor = cursor
         version(cursor=cursor, condition=[9, 0, 1])
 
@@ -221,10 +214,7 @@ cursor: DBcursor, optional
                 "nbins": 1000,
             }
         )
-        if not (cursor):
-            cursor = read_auto_connect().cursor()
-        else:
-            check_cursor(cursor)
+        cursor = check_cursor(cursor)[0]
         self.cursor = cursor
         version(cursor=cursor, condition=[8, 1, 1])
 
@@ -259,9 +249,6 @@ cursor: DBcursor, optional
                 "nbins": 1000,
             }
         )
-        if not (cursor):
-            cursor = read_auto_connect().cursor()
-        else:
-            check_cursor(cursor)
+        cursor = check_cursor(cursor)[0]
         self.cursor = cursor
         version(cursor=cursor, condition=[9, 0, 1])
