@@ -62,6 +62,7 @@ from matplotlib.colors import LinearSegmentedColormap
 from verticapy.utilities import *
 from verticapy.toolbox import *
 from verticapy.errors import *
+import verticapy
 
 #
 ##
@@ -151,16 +152,17 @@ def bar(
         ax.set_yticklabels(new_z, rotation=0)
     else:
         ax.set_yticks([elem - h / 2 / 0.94 for elem in x])
-    if method == "density":
+    if method.lower() == "density":
         ax.set_xlabel("Density")
-        ax.set_title("Distribution of {}".format(vdf.alias))
-    elif (method in ["avg", "min", "max", "sum"] or "%" in method) and (of != None):
+    elif (method.lower() in ["avg", "min", "max", "sum"] or "%" == method[-1]) and (
+        of != None
+    ):
         aggregate = "{}({})".format(method.upper(), of)
         ax.set_xlabel(aggregate)
-        ax.set_title("{} group by {}".format(aggregate, vdf.alias))
-    else:
+    elif method.lower() == "count":
         ax.set_xlabel("Frequency")
-        ax.set_title("Count by {}".format(vdf.alias))
+    else:
+        ax.set_xlabel(method)
     return ax
 
 
@@ -240,21 +242,18 @@ def bar2D(
             )
             ax.set_yticklabels(all_columns[0][1:m])
         ax.set_ylabel(columns[0])
-        if method == "mean":
+        if method.lower() == "mean":
             method = "avg"
-        if method == "density":
+        if method.lower() == "mean":
+            method = "avg"
+        if method.lower() == "density":
             ax.set_xlabel("Density")
-            ax.set_title(
-                "Distribution of {} group by {}".format(columns[0], columns[1])
-            )
-        elif (method in ["avg", "min", "max", "sum"] or "%" in method) and (of != None):
+        elif (method.lower() in ["avg", "min", "max", "sum"]) and (of != None):
             ax.set_xlabel("{}({})".format(method, of))
-            ax.set_title(
-                "{}({}) of {} group by {}".format(method, of, columns[0], columns[1])
-            )
-        else:
+        elif method.lower() == "count":
             ax.set_xlabel("Frequency")
-            ax.set_title("Count by {} group by {}".format(columns[0], columns[1]))
+        else:
+            ax.set_xlabel(method)
     else:
         total = [0 for item in range(1, m)]
         for i in range(1, n):
@@ -303,10 +302,6 @@ def bar2D(
         ax.set_yticks([elem for elem in range(n_groups)])
         ax.set_yticklabels(all_columns[0][1:m])
         ax.set_ylabel(columns[0])
-        ax.set_xlabel("Density per category")
-        ax.set_title(
-            "Distribution per category of {} group by {}".format(columns[0], columns[1])
-        )
     ax.legend(title=columns[1], loc="center left", bbox_to_anchor=[1, 0.5])
     return ax
 
@@ -355,7 +350,6 @@ def boxplot(
         for patch in box["boxes"]:
             patch.set_facecolor("#FE5016")
         ax.set_axisbelow(True)
-        ax.set_title("BoxPlot of {}".format(vdf.alias))
         return ax
     # MULTI BOXPLOT
     else:
@@ -518,7 +512,6 @@ def boxplot(
                 patch_artist=True,
             )
             ax.set_xticklabels(labels, rotation=90)
-            ax.set_title("BoxPlot of {} group by {}".format(vdf.alias, by))
             colors = gen_colors()
             for median in box["medians"]:
                 median.set(
@@ -573,7 +566,6 @@ def boxplot2D(vdf, columns: list = [], ax=None):
                 patch_artist=True,
             )
             ax.set_xticklabels(columns, rotation=90)
-            ax.set_title("Multi BoxPlot of the vDataFrame")
             colors = gen_colors()
             for median in box["medians"]:
                 median.set(
@@ -642,7 +634,6 @@ def bubble(
                 ax.set_xlim(bbox[0], bbox[1])
                 ax.set_ylim(bbox[2], bbox[3])
             ax.imshow(im, extent=bbox)
-        ax.set_title("Bubble Plot of {} vs {}".format(columns[0], columns[1]))
         ax.set_ylabel(columns[1])
         ax.set_xlabel(columns[0])
         scatter = ax.scatter(column1, column2, color=colors[0], s=size, alpha=0.5)
@@ -745,7 +736,6 @@ def bubble(
             loc="center left",
             title=columns[2]
         )
-        ax.set_title("Scatter Plot of {} vs {}".format(columns[0], columns[1]))
         ax.set_xlabel(columns[0])
         ax.set_ylabel(columns[1])
         leg2 = ax.legend(
@@ -784,14 +774,11 @@ def cmatrix(
 ):
     if is_vector:
         is_vector = True
-        vector = []
-        for elem in matrix_array:
-            if len(elem) == 1:
-                vector += elem
-            else:
-                raise
-        vector = [vector]
-        matrix_array = vector
+        vector = [elem for elem in matrix[1]]
+        matrix_array = vector[1:]
+        for i in range(len(matrix_array)):
+            matrix_array[i] = round(float(matrix_array[i]), mround)
+        matrix_array = [matrix_array]
         m, n = n, m
         x_label, y_label = y_label, x_label
         columns_x, columns_y = columns_y, columns_x
@@ -810,24 +797,26 @@ def cmatrix(
             columns_x.reverse()
     if not (ax):
         fig, ax = plt.subplots()
-        if (isnotebook() and not(inverse)) or is_pivot:
+        if (isnotebook() and not (inverse)) or is_pivot:
             fig.set_size_inches(min(m, 500), min(n, 500))
         else:
             fig.set_size_inches(8, 6)
     else:
         fig = plt
     ax.set_title(title)
-    if ((vmax == 1) and vmin in [0, -1]) and not(extent):
+    if ((vmax == 1) and vmin in [0, -1]) and not (extent):
         im = ax.imshow(
             matrix_array, cmap=cmap, interpolation=interpolation, vmax=vmax, vmin=vmin
         )
     else:
         try:
-            im = ax.imshow(matrix_array, cmap=cmap, interpolation=interpolation, extent=extent)
+            im = ax.imshow(
+                matrix_array, cmap=cmap, interpolation=interpolation, extent=extent
+            )
         except:
             im = ax.imshow(matrix_array, cmap=cmap, interpolation=interpolation,)
     fig.colorbar(im, ax=ax).set_label(colorbar)
-    if not(extent):
+    if not (extent):
         ax.set_yticks([i for i in range(0, n)])
         ax.set_xticks([i for i in range(0, m)])
         ax.set_xticklabels(columns_y, rotation=90)
@@ -852,29 +841,46 @@ def compute_plot_variables(
     h: float = 0,
     pie: bool = False,
 ):
-    if method == "median":
+    other_columns = ""
+    if method.lower() == "median":
         method = "50%"
-    elif method == "mean":
+    elif method.lower() == "mean":
         method = "avg"
     if (
-        (method in ["avg", "min", "max", "sum"]) or (method and method[-1] == "%")
+        method.lower() not in ["avg", "min", "max", "sum", "density", "count"]
+        and "%" != method[-1]
+    ) and of:
+        raise ParameterError(
+            "Parameter 'of' must be empty when using customized aggregations."
+        )
+    if (
+        (method.lower() in ["avg", "min", "max", "sum"])
+        or (method.lower() and method[-1] == "%")
     ) and (of):
-        if method in ["avg", "min", "max", "sum"]:
+        if method.lower() in ["avg", "min", "max", "sum"]:
             aggregate = "{}({})".format(method.upper(), str_column(of))
-            others_aggregate = method
         elif method and method[-1] == "%":
             aggregate = "APPROXIMATE_PERCENTILE({} USING PARAMETERS percentile = {})".format(
                 str_column(of), float(method[0:-1]) / 100
             )
-            others_aggregate = method
         else:
-            raise MissingColumn("The column '" + of + "' doesn't exist")
-    elif method in ["density", "count"]:
+            raise ParameterError(
+                "The parameter 'method' must be in [avg|mean|min|max|sum|median|q%] or a customized aggregation. Found {}.".format(
+                    method
+                )
+            )
+    elif method.lower() in ["density", "count"]:
         aggregate = "count(*)"
-        others_aggregate = "sum"
+    elif isinstance(method, str):
+        aggregate = method
+        other_columns = ", " + ", ".join(
+            vdf.parent.get_columns(exclude_columns=[vdf.alias])
+        )
     else:
         raise ParameterError(
-            "The parameter 'method' must be in [avg|mean|min|max|sum|median|q%]"
+            "The parameter 'method' must be in [avg|mean|min|max|sum|median|q%] or a customized aggregation. Found {}.".format(
+                method
+            )
         )
     # depending on the cardinality, the type, the vColumn can be treated as categorical or not
     cardinality, count, is_numeric, is_date, is_categorical = (
@@ -911,7 +917,9 @@ def compute_plot_variables(
                 )
                 if of:
                     enum_trans += " , " + of
-                table = "(SELECT {} FROM {}) enum_table".format(enum_trans, table)
+                table = "(SELECT {} FROM {}) enum_table".format(
+                    enum_trans + other_columns, table
+                )
             query = "(SELECT {} AS {}, {} FROM {} GROUP BY {} ORDER BY 2 DESC LIMIT {})".format(
                 convert_special_type(vdf.category(), True, vdf.alias),
                 vdf.alias,
@@ -922,14 +930,16 @@ def compute_plot_variables(
             )
             if cardinality > max_cardinality:
                 query += (
-                    " UNION (SELECT 'Others', {}(count) FROM (SELECT {} AS count FROM {} "
-                    + "GROUP BY {} ORDER BY {} DESC OFFSET {}) y LIMIT 1) ORDER BY 2 DESC"
+                    " UNION (SELECT 'Others', {} FROM {} WHERE {} NOT IN "
+                    + "(SELECT {} FROM {} GROUP BY {} ORDER BY {} DESC LIMIT {}))"
                 )
                 query = query.format(
-                    others_aggregate,
                     aggregate,
                     table,
                     vdf.alias,
+                    vdf.alias,
+                    table,
+                    vdf.alias,
                     aggregate,
                     max_cardinality,
                 )
@@ -940,7 +950,7 @@ def compute_plot_variables(
         z = [item[0] for item in query_result]
         y = (
             [item[1] / float(count) if item[1] != None else 0 for item in query_result]
-            if (method == "density")
+            if (method.lower() == "density")
             else [item[1] if item[1] != None else 0 for item in query_result]
         )
         x = [0.4 * i + 0.2 for i in range(0, len(y))]
@@ -967,7 +977,7 @@ def compute_plot_variables(
         x = [float(item[0]) for item in query_result]
         y = (
             [item[1] / float(count) for item in query_result]
-            if (method == "density")
+            if (method.lower() == "density")
             else [item[1] for item in query_result]
         )
         query = ""
@@ -998,7 +1008,7 @@ def compute_plot_variables(
         query_result = vdf.parent._VERTICAPY_VARIABLES_["cursor"].fetchall()
         y = (
             [item[1] / float(count) for item in query_result]
-            if (method == "density")
+            if (method.lower() == "density")
             else [item[1] for item in query_result]
         )
         x = [float(item[0]) + h / 2 for item in query_result]
@@ -1009,34 +1019,39 @@ def compute_plot_variables(
 
 # ---#
 def gen_cmap():
-    cm1 = LinearSegmentedColormap.from_list("vml", ["#FFFFFF", "#FE5016"], N=1000)
+    cm1 = LinearSegmentedColormap.from_list("vml", ["#FFFFFF", gen_colors()[0]], N=1000)
     cm2 = LinearSegmentedColormap.from_list(
-        "vml", ["#263133", "#FFFFFF", "#FE5016"], N=1000
+        "vml", [gen_colors()[1], "#FFFFFF", gen_colors()[0]], N=1000
     )
     return (cm1, cm2)
 
 
 # ---#
 def gen_colors():
-    colors = [
-        "#FE5016",
-        "#263133",
-        "#0073E7",
-        "#19A26B",
-        "#FCDB1F",
-        "#2A6A74",
-        "#861889",
-        "#00B4E0",
-        "#90EE90",
-        "#FF7F50",
-        "#B03A89",
-    ]
-    all_colors = [item for item in plt_colors.cnames]
-    shuffle(all_colors)
-    for c in all_colors:
-        if c not in colors:
-            colors += [c]
-    return colors
+    if not (verticapy.options["colors"]) or not (
+        isinstance(verticapy.options["colors"], list)
+    ):
+        colors = [
+            "#FE5016",
+            "#263133",
+            "#0073E7",
+            "#19A26B",
+            "#FCDB1F",
+            "#2A6A74",
+            "#861889",
+            "#00B4E0",
+            "#90EE90",
+            "#FF7F50",
+            "#B03A89",
+        ]
+        all_colors = [item for item in plt_colors.cnames]
+        shuffle(all_colors)
+        for c in all_colors:
+            if c not in colors:
+                colors += [c]
+        return colors
+    else:
+        return verticapy.options["colors"]
 
 
 # ---#
@@ -1056,32 +1071,32 @@ def hexbin(
         raise ParameterError(
             "The parameter 'columns' must be exactly of size 2 to draw the hexbin"
         )
-    if method == "mean":
+    if method.lower() == "mean":
         method = "avg"
     if (
-        (method in ["avg", "min", "max", "sum"])
+        (method.lower() in ["avg", "min", "max", "sum"])
         and (of)
         and ((of in vdf.get_columns()) or (str_column(of) in vdf.get_columns()))
     ):
         aggregate = "{}({})".format(method, of)
         others_aggregate = method
-        if method == "avg":
+        if method.lower() == "avg":
             reduce_C_function = statistics.mean
-        elif method == "min":
+        elif method.lower() == "min":
             reduce_C_function = min
-        elif method == "max":
+        elif method.lower() == "max":
             reduce_C_function = max
-        elif method == "sum":
+        elif method.lower() == "sum":
             reduce_C_function = sum
-    elif method in ("count", "density"):
+    elif method.lower() in ("count", "density"):
         aggregate = "count(*)"
         reduce_C_function = sum
     else:
         raise ParameterError(
-            "The parameter 'method' must be in [avg|mean|min|max|sum|median|q%]"
+            "The parameter 'method' must be in [avg|mean|min|max|sum|median]"
         )
     count = vdf.shape()[0]
-    if method == "density":
+    if method.lower() == "density":
         over = "/" + str(float(count))
     else:
         over = ""
@@ -1123,7 +1138,6 @@ def hexbin(
             ax.set_xlim(bbox[0], bbox[1])
             ax.set_ylim(bbox[2], bbox[3])
         ax.imshow(im, extent=bbox)
-    ax.set_title("Hexbin of {} vs {}".format(columns[0], columns[1]))
     ax.set_ylabel(columns[1])
     ax.set_xlabel(columns[0])
     imh = ax.hexbin(
@@ -1137,7 +1151,7 @@ def hexbin(
         mincnt=1,
         edgecolors=None,
     )
-    if method == "density":
+    if method.lower() == "density":
         fig.colorbar(imh).set_label(method)
     else:
         fig.colorbar(imh).set_label(aggregate)
@@ -1180,18 +1194,17 @@ def hist(
     else:
         ax.set_xticks([elem - h / 2 / 0.94 for elem in x])
         ax.set_xticklabels([elem - h / 2 / 0.94 for elem in x], rotation=90)
-    if method == "density":
+    if method.lower() == "density":
         ax.set_ylabel("Density")
-        ax.set_title("Distribution of {}".format(vdf.alias))
-    elif (method in ["avg", "min", "max", "sum", "mean"] or ("%" in method)) and (
-        of != None
-    ):
+    elif (
+        method.lower() in ["avg", "min", "max", "sum", "mean"] or ("%" == method[-1])
+    ) and (of != None):
         aggregate = "{}({})".format(method, of)
-        ax.set_ylabel(aggregate)
-        ax.set_title("{} group by {}".format(aggregate, vdf.alias))
-    else:
+        ax.set_ylabel(method)
+    elif method.lower() == "count":
         ax.set_ylabel("Frequency")
-        ax.set_title("Count by {}".format(vdf.alias))
+    else:
+        ax.set_ylabel(method)
     return ax
 
 
@@ -1270,17 +1283,14 @@ def hist2D(
     ax.set_xlabel(columns[0])
     if method.lower() == "mean":
         method = "avg"
-    if method == "density":
+    if method.lower() == "density":
         ax.set_ylabel("Density")
-        ax.set_title("Distribution of {} group by {}".format(columns[0], columns[1]))
-    elif (method in ["avg", "min", "max", "sum"]) and (of != None):
+    elif (method.lower() in ["avg", "min", "max", "sum"]) and (of != None):
         ax.set_ylabel("{}({})".format(method, of))
-        ax.set_title(
-            "{}({}) of {} group by {}".format(method, of, columns[0], columns[1])
-        )
-    else:
+    elif method.lower() == "count":
         ax.set_ylabel("Frequency")
-        ax.set_title("Count by {} group by {}".format(columns[0], columns[1]))
+    else:
+        ax.set_ylabel(method)
     ax.legend(title=columns[1], loc="center left", bbox_to_anchor=[1, 0.5])
     return ax
 
@@ -1325,15 +1335,17 @@ def multiple_hist(
                     )
                     warnings.warn(warning_message, Warning)
         ax.set_xlabel(", ".join(all_columns))
-        if method == "density":
+        if method.lower() == "density":
             ax.set_ylabel("Density")
-        elif (method in ["avg", "min", "max", "sum", "mean"] or ("%" in method)) and (
-            of
-        ):
+        elif (
+            method.lower() in ["avg", "min", "max", "sum", "mean"]
+            or ("%" == method[-1])
+        ) and (of):
             ax.set_ylabel(method + "(" + of + ")")
-        else:
+        elif method.lower() == "count":
             ax.set_ylabel("Frequency")
-        ax.set_title("Multiple Histograms")
+        else:
+            ax.set_ylabel(method)
         ax.legend(title="columns", loc="center left", bbox_to_anchor=[1, 0.5])
         return ax
 
@@ -1400,7 +1412,6 @@ def multi_ts_plot(
             label=columns[i],
             linewidth=2,
         )
-    ax.set_title("Multi Plot of the vDataFrame")
     for tick in ax.get_xticklabels():
         tick.set_rotation(90)
     ax.set_xlabel(order_by)
@@ -1432,7 +1443,7 @@ def pie(
         if (item < 0.05) or ((item > 1) and (float(item) / float(total_count) < 0.05)):
             current_explode = min(0.9, current_explode * 1.4)
             explode[idx] = current_explode
-    if method == "density":
+    if method.lower() == "density":
         autopct = "%1.1f%%"
     else:
 
@@ -1448,8 +1459,8 @@ def pie(
 
             return my_autopct
 
-        if (method in ["sum", "count"]) or (
-            (method in ["min", "max"]) and (vdf.parent[of].category == "int")
+        if (method.lower() in ["sum", "count"]) or (
+            (method.lower() in ["min", "max"]) and (vdf.parent[of].category == "int")
         ):
             category = "int"
         else:
@@ -1475,20 +1486,23 @@ def pie(
         startangle=290,
         explode=explode,
     )
-    if method == "density":
-        ax.set_title("Distribution of {}".format(vdf.alias))
-    elif (method in ["avg", "min", "max", "sum"] or "%" in method) and (of != None):
-        aggregate = "{}({})".format(method, of)
-        ax.set_title("{} group by {}".format(aggregate, vdf.alias))
+    if method.lower() == "density":
+        ax.set_title("Density")
+    elif (
+        method.lower() in ["avg", "min", "max", "sum", "mean"] or ("%" == method[-1])
+    ) and (of):
+        ax.set_title(method + "(" + of + ")")
+    elif method.lower() == "count":
+        ax.set_title("Frequency")
     else:
-        ax.set_title("Count by {}".format(vdf.alias))
+        ax.set_title(method)
     return ax
 
 
 # ---#
 def pivot_table(
     vdf,
-    columns,
+    columns: list,
     method: str = "count",
     of: str = "",
     h: tuple = (None, None),
@@ -1501,20 +1515,30 @@ def pivot_table(
     return_ax: bool = False,
     extent: list = [],
 ):
-    if method == "median":
+    other_columns = ""
+    if method.lower() == "median":
         method = "50%"
-    elif method == "mean":
+    elif method.lower() == "mean":
         method = "avg"
+    if (
+        method.lower() not in ["avg", "min", "max", "sum", "density", "count"]
+        and "%" != method[-1]
+    ) and of:
+        raise ParameterError(
+            "Parameter 'of' must be empty when using customized aggregations."
+        )
     if (method.lower() in ["avg", "min", "max", "sum"]) and (of):
         aggregate = "{}({})".format(method.upper(), str_column(of))
-        others_aggregate = method
-    elif method and method[-1] == "%":
+    elif method.lower() and method[-1] == "%":
         aggregate = "APPROXIMATE_PERCENTILE({} USING PARAMETERS percentile = {})".format(
             str_column(of), float(method[0:-1]) / 100
         )
     elif method.lower() in ["density", "count"]:
         aggregate = "COUNT(*)"
-        others_aggregate = "sum"
+    elif isinstance(method, str):
+        aggregate = method
+        other_columns = vdf.get_columns(exclude_columns=columns)
+        other_columns = ", " + ", ".join(other_columns)
     else:
         raise ParameterError(
             "The parameter 'method' must be in [count|density|avg|mean|min|max|sum|q%]"
@@ -1623,34 +1647,48 @@ def pivot_table(
         return to_tablesample(query, vdf._VERTICAPY_VARIABLES_["cursor"])
     alias = ", " + str_column(of) + " AS " + str_column(of) if of else ""
     aggr = ", " + of if (of) else ""
-    subtable = "(SELECT {} AS {}, {} AS {}{} FROM {}{}) pivot_table".format(
+    subtable = "(SELECT {} AS {}, {} AS {}{}{} FROM {}{}) pivot_table".format(
         all_columns[0],
         columns[0],
         all_columns[1],
         columns[1],
         alias,
+        other_columns,
         vdf.__genSQL__(),
         where,
     )
     if is_column_date[0] and not (is_column_date[1]):
-        subtable = "(SELECT {} AS {}, {}{} FROM {}{}) pivot_table_date".format(
-            timestampadd[0], columns[0], columns[1], aggr, subtable, where
+        subtable = "(SELECT {} AS {}, {}{}{} FROM {}{}) pivot_table_date".format(
+            timestampadd[0],
+            columns[0],
+            columns[1],
+            aggr,
+            other_columns,
+            subtable,
+            where,
         )
     elif is_column_date[1] and not (is_column_date[0]):
-        subtable = "(SELECT {}, {} AS {}{} FROM {}{}) pivot_table_date".format(
-            columns[0], timestampadd[1], columns[1], aggr, subtable, where
+        subtable = "(SELECT {}, {} AS {}{}{} FROM {}{}) pivot_table_date".format(
+            columns[0],
+            timestampadd[1],
+            columns[1],
+            aggr,
+            other_columns,
+            subtable,
+            where,
         )
     elif is_column_date[1] and is_column_date[0]:
-        subtable = "(SELECT {} AS {}, {} AS {}{} FROM {}{}) pivot_table_date".format(
+        subtable = "(SELECT {} AS {}, {} AS {}{}{} FROM {}{}) pivot_table_date".format(
             timestampadd[0],
             columns[0],
             timestampadd[1],
             columns[1],
             aggr,
+            other_columns,
             subtable,
             where,
         )
-    over = "/" + str(vdf.shape()[0]) if (method == "density") else ""
+    over = "/" + str(vdf.shape()[0]) if (method.lower() == "density") else ""
     cast = []
     for column in columns:
         cast += [convert_special_type(vdf[column].category(), True, column)]
@@ -1731,7 +1769,7 @@ def pivot_table(
             vmax=max(all_count),
             vmin=min(all_count),
             cmap=cmap,
-            title="Pivot Table of " + columns[0] + " vs " + columns[1],
+            title="",
             colorbar=aggregate,
             x_label=columns[1],
             y_label=columns[0],
@@ -1770,8 +1808,9 @@ def scatter_matrix(vdf, columns: list = []):
             nrows=n, ncols=n, figsize=(min(int(n / 1.1), 500), min(int(n / 1.1), 500))
         )
     )
-    query = "SELECT {}, RANDOM() AS rand FROM {} WHERE __verticapy_split__ < 0.5 ORDER BY rand LIMIT 1000".format(
-        ", ".join(columns), vdf.__genSQL__(True)
+    random_func = random_function()
+    query = "SELECT {}, {} AS rand FROM {} WHERE __verticapy_split__ < 0.5 ORDER BY rand LIMIT 1000".format(
+        ", ".join(columns), random_func, vdf.__genSQL__(True)
     )
     all_scatter_points = vdf.__executeSQL__(
         query=query, title="Select random points to draw the scatter plot"
@@ -1802,7 +1841,7 @@ def scatter_matrix(vdf, columns: list = []):
                 axes[i, j].scatter(
                     all_scatter_columns[j],
                     all_scatter_columns[i],
-                    color="#263133",
+                    color=gen_colors()[1],
                     s=4,
                     marker="o",
                 )
@@ -1874,7 +1913,6 @@ def scatter2D(
             ax.imshow(im, extent=bbox)
             ax.grid()
             ax.set_axisbelow(True)
-        ax.set_title("Scatter Plot of {} vs {}".format(columns[0], columns[1]))
         ax.set_ylabel(columns[1])
         ax.set_xlabel(columns[0])
         ax.scatter(column1, column2, color=colors[0], s=14)
@@ -2000,7 +2038,6 @@ def scatter2D(
         for idx, item in enumerate(all_categories):
             if len(str(item)) > 20:
                 all_categories[idx] = str(item)[0:20] + "..."
-        ax.set_title("Scatter Plot of {} vs {}".format(columns[0], columns[1]))
         ax.set_xlabel(columns[0])
         ax.set_ylabel(columns[1])
         ax.legend(
@@ -2066,11 +2103,6 @@ def scatter3D(
                 if isnotebook():
                     plt.figure(figsize=(8, 6))
                 ax = plt.axes(projection="3d")
-            ax.set_title(
-                "Scatter Plot of {} vs {} vs {}".format(
-                    columns[0], columns[1], columns[2]
-                )
-            )
             ax.scatter(column1, column2, column3, color=colors[0])
             ax.set_xlabel(columns[0])
             ax.set_ylabel(columns[1])
@@ -2188,11 +2220,6 @@ def scatter3D(
             for idx, item in enumerate(all_categories):
                 if len(str(item)) > 20:
                     all_categories[idx] = str(item)[0:20] + "..."
-            ax.set_title(
-                "Scatter Plot of {} vs {} vs {}".format(
-                    columns[0], columns[1], columns[2]
-                )
-            )
             ax.set_xlabel(columns[0])
             ax.set_ylabel(columns[1])
             ax.set_zlabel(columns[2])
@@ -2260,7 +2287,6 @@ def ts_plot(
             ax.fill_between(order_by_values, column_values, facecolor=color)
         else:
             area_label = ""
-        ax.set_title("{}Plot of {} vs {}".format(area_label, vdf.alias, order_by))
         for tick in ax.get_xticklabels():
             tick.set_rotation(90)
         ax.set_xlabel(order_by)
@@ -2312,7 +2338,6 @@ def ts_plot(
             ax.grid()
         for idx, elem in enumerate(all_data):
             ax.plot(elem[0], elem[1], color=colors[idx % len(colors)], label=elem[2])
-        ax.set_title("Plot of {} vs {}".format(vdf.alias, order_by))
         ax.set_xlabel(order_by)
         ax.set_ylabel(vdf.alias)
         ax.legend(title=by, loc="center left", bbox_to_anchor=[1, 0.5])
