@@ -11,34 +11,41 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import pytest
-from verticapy import vDataFrame
-from verticapy import drop_table
+import pytest, warnings
+from verticapy import vDataFrame, drop_table
+
+from verticapy import set_option
+
+set_option("print_info", False)
 
 
 @pytest.fixture(scope="module")
 def titanic_vd(base):
     from verticapy.learn.datasets import load_titanic
 
-    titanic = load_titanic(cursor = base.cursor)
-    titanic.set_display_parameters(print_info=False)
+    titanic = load_titanic(cursor=base.cursor)
     yield titanic
-    drop_table(name="public.titanic", cursor=base.cursor)
+    with warnings.catch_warnings(record=True) as w:
+        drop_table(name="public.titanic", cursor=base.cursor)
 
 
 class TestvDFCreate:
     def test_creating_vDF_using_input_relation(self, base, titanic_vd):
-        tvdf = vDataFrame(input_relation = 'public.titanic', cursor = base.cursor)
+        tvdf = vDataFrame(input_relation="public.titanic", cursor=base.cursor)
 
         assert tvdf["pclass"].count() == 1234
 
     def test_creating_vDF_using_input_relation_schema(self, base, titanic_vd):
-        tvdf = vDataFrame(input_relation = 'titanic', schema = 'public', cursor = base.cursor)
+        tvdf = vDataFrame(input_relation="titanic", schema="public", cursor=base.cursor)
 
         assert tvdf["pclass"].count() == 1234
 
     def test_creating_vDF_using_input_relation_vcolumns(self, base, titanic_vd):
-        tvdf = vDataFrame(input_relation = 'public.titanic', usecols = ["age", "survived"], cursor = base.cursor)
+        tvdf = vDataFrame(
+            input_relation="public.titanic",
+            usecols=["age", "survived"],
+            cursor=base.cursor,
+        )
 
         assert tvdf["survived"].count() == 1234
 
